
CheckViaSmallGroups := function(n, f, p, q)
    local a, i, G, b;

    if SMALL_AVAILABLE(p^n*q) = fail then return; fi;

    # small groups
    a := 0;
    for i in [1..NumberSmallGroups(p^n*q)] do
        G := SmallGroup(p^n*q, i);
        if not IsNilpotent(G) and IsNormal(G, SylowSubgroup(G, p)) then
            a := a+1;
        fi;
    od;

    # by function
    b := EvalPNQPorcPoly(f, p, q);
    if a<>b then Error("hier"); fi;
end;

CheckDimViaSG := function(n)
    local ff, pr, P, Q;
    ff := NumberPNQNormalPSylow(n);
    if ff = fail then return fail; fi;
    pr := Filtered([2..100], IsPrime);
    for P in pr do
        for Q in pr do
            if P <> Q and P > n-1 then
                CheckViaSmallGroups(n, ff, P, Q);
            fi;
        od;
    od;
end;

InducedActionAut := function(G)
    local A, d, P, g, m, a, h;
    A := AutomorphismGroup(G);
    d := RankPGroup(G);
    P := PrimePGroup(G);
    g := Pcgs(G);
    m := [];
    for a in GeneratorsOfGroup(A) do
        h := List(g{[1..d]}, x -> ExponentsOfPcElement(g, Image(a,x)){[1..d]});
        Add(m, h * One(GF(P)));
    od;
    return Group(m, IdentityMat(d,GF(P)));
end;

NrRationalClassesQElements := function( G, q )
    local S, c, N, U, o, l, i, j, k;

    Print("Sylow for ",q,"\n");
    S := SylowSubgroup(G, q);
    if Size(S) = q then return 1; fi;

    Print("  get rat classes for ",q," in group of size ",Size(S)," \n");
    if q = 2 then 
        c := ConjugacyClasses(S);
    else
        c := RationalClasses(S);
    fi;
    c := List(c, Representative);
    c := Filtered(c, x -> Order(x)=q);
    if Length(c) = 1 then return 1; fi;
    Print("  has ",Length(c)," classes \n");

    N := Normalizer(G, Subgroup(S, c));
    U := List(c, x -> Subgroup(G, [x]));
    o := Orbits(N, U);
    l := List(o, x -> GeneratorsOfGroup(x[1])[1]);
    Print("  reduced to ",Length(o)," orbits under N \n");

    for i in [1..Length(l)] do
        for j in [i+1..Length(l)] do
            if l[i] <> false and l[j] <> false then 
                for k in [1..q-1] do
                    if IsConjugate(G, l[i], l[j]^k) then 
                        l[j] := false;
                        break;
                    fi;
                od;
            fi;
        od;
    od;

    l := Filtered(l, x -> x <> false);
    Print("  reduced to ",Length(l)," orbits under G \n");
    return Length(l);
end; 

InvestigateLiePRing := function( L, P )
    local S, r, i, G, A, c, Q, u, qs, B;

    # specialise
    S := LiePRingsInFamily(L, P);
    if S = [] then return S; fi;

    r := List(S, x -> []);
    for i in [1..Length(S)] do
        G := PGroupByLiePRing(S[i]);
        A := InducedActionAut(G);
        B := Image(IsomorphismPermGroup(A));
        qs := Set(Factors(Size(B))); 
        qs := Filtered(qs, x -> x <> P);
        for Q in qs do
            c := NrRationalClassesQElements(B, Q);
            Add(r[i], [Q,c]);
        od;
    od;
    return r;
end;

CheckLiePRing := function( L, P, qs )
    local x, f, S, G, A, B, Q, u, v;

    x := IndeterminateByName("x");
    if x in ParametersOfLiePRing(L) then return fail; fi;

    # get function
    f := ZupposOfLiePRing(L);
    if f = fail then return fail; fi;

    # specialise
    S := SpecialisePrimeOfLiePRing(L, P);
    if S = fail then return fail; fi;

    # get classes
    G := PGroupByLiePRing(S);
    A := InducedActionAut(G);
    B := Image(IsomorphismPermGroup(A));

    # loop
    for Q in qs do
        if Q <> P then 
            Print("   process prime ",Q,"\n");
            u := NrRationalClassesQElements(B, Q);
            v := EvalPNQPorcPoly(f, P, Q);
            if u <> v then Error(Q); fi;
        fi;
    od;

    return true;
end;

# ab 69 nur noch P in [5,7,11]

CheckDimViaZuppos := function(n,s)
    local L, P, i, Q, H;
    L := LiePRingsByLibrary(n);
    #Q := [2,3,5,7,11,13,17,19];
    Q := [2];
    for i in [s..Length(L)] do
        for P in [5,7,11] do
            Print("process ring ",i," with p=",P,"\n");
            CheckLiePRing(L[i], P, Q);
        od;
    od;
end;
      
