IsElementaryAbelianLiePRing := function ( L )
local  S, n, p, l, i, e, j;
    S := SCTable( Zero( L ) );
    n := DimensionOfLiePRing( L );
    p := PrimeOfLiePRing( L );
    l := BasisOfLiePRing( L );
    for i  in [ 1 .. n ]  do
        e := ExponentsLPR( L, p * l[i] );
        if e <> 0 * e  then
            return false;
        fi;
    od;
    for i  in [ 1 .. n ]  do
        for j  in [ 1 .. i - 1 ]  do
            e := ExponentsLPR( L, l[i] * l[j] );
            if e <> 0 * e  then
                return false;
            fi;
        od;
    od;
    return true;
end;



LiePAutosOnQuotient := function( L, C )
local A, bL, bC, pos, max;

	A := FindAutos(L);
	bL := BasisOfLiePRing(L);
	bC := BasisOfLiePRing(C);
	
	# assuming that the basis of the quotient lies nicely in the basis
	# of the Lie p-ring read off the autos
	pos := Positions(List(bL, x -> x in bC), false);
	
	max := Minimum(MinimalGeneratorNumberOfLiePRing(L), Maximum(pos));	

	return rec( auto := A.auto{[1..max]}{pos}, eqns := A.eqns );
end;



SumOfIndices520 := function()
    local a, b, c;

    # 5.20
    # if x = 1, then this is GL(2,p)
    # if x <> 1 then
    #   A33 <> 0 yields A32=A23=0 and D A22 A33 = 1, thus (p-1)^2
    #   A33 = 0 yields D A23 A32 = 1 and A22 = 0 and x=-1, thus (p-1)^2
    # return "GL(2,p) if x=1, 2(p-1)^2 if x=-1, (p-1)^2 otherwise";

    # x = 1 (group is isomorphic to GL(2,p))
    a := 2*p-1;

    # x = -1 (group has the form C_{p-1}^2 split C_2)
    b := 2*p;

    # x <> 1,-1 (group is cyclic of the form C_{p-1}^2), (p-3)/2 cases 
    c := ((p-3)/2)*(4*p-1);

    return [a+b+c];
end;


SumOfIndices542 := function()
    local a, b, c;

    # 5.42
    # if x = 1 then this is SL(2,p)
    # if x <> 1 then
    #   A22 <> 0 yields A21 = A12 = 0 and A11 A22 = 1, thus (p-1)
    #   A22 = 0 yields x A12 A21 = -1 and A11 = 0 and x=-1, thus (p-1)
    # return "SL(2,p) if x=1, 2(p-1) if x =-1, (p-1) otherwise";

    # x = 1 (group is isomorphic to SL(2,p))
    a := 1;

    # x = -1 (group has the form C_{p-1} split C_2)
    b := p;

    # x <> 1,-1 (group is cyclic of the form C_{p-1}), (p-3)/2 cases
    c := ((p-3)/2)*(p+1);

    return [a+b+c];
end;



#############################################################################
##
## SumOfIndicesOfLiePRingNC
##
SumOfIndicesOfLiePRingNC := function( L )
    local pos, C, Q, R, Rpr, d, n, z, A, i, j, D, a, e, b, B, SL, GL, EAQ;

	# check for trivial cases 
	# cyclic with full gl
	pos := Position(["1.0", "2.0", "3.0", "4.0", "5.0"], LibraryName(L));
	
	if IsInt(pos) then
		return ListWithIdenticalEntries(pos, 1);
	fi;

	# elementary abelian with full gl
	if LibraryName(L) in ["1.1", "2.1", "3.1", "4.1", "5.1"] then
		return [1];
	fi;

    # set up and find autos
	C := LiePCommutator(L, L);

	if DimensionOfLiePRing(C) > 0 then
		R := LiePAutosOnQuotient(L, C);
		Q := LiePQuotient(L, C);
	else
		R := FindAutos(L);
		Q := L;
	fi;

    d := MinimalGeneratorNumberOfLiePRing(L);
    z := p^0;
	Rpr := ShallowCopy(R);
    Rpr.auto := R.auto{[1..d]}{[1..d]} * z;
    R.auto := R.auto * z;


    # get matrix of indeterminates 
    A := MutableNullMat(5,5);
    for i in [1..5] do
        for j in [1..5] do
            A[i][j] := Indeterminate(Rationals, Concatenation("A",
                        String(i), String(j)));
        od;
    od;

    # check whether quotient is elementary abelian + GL / SL cases
	EAQ := IsElementaryAbelianLiePRing(Q);
    D := Indeterminate(Rationals, "D");
    a := Determinant(Rpr.auto);
    GL := (Rpr.eqns = [D*a-1] or Rpr.eqns = [1-D*a]);
    SL := (Rpr.eqns = [a-1] or Rpr.eqns = [1-a]);

	#Print("EAQ: ", EAQ, "    GL: ", GL, "   SL: ", SL, "\n");

	# quotient elem. ab. dim 2 
	# orb reps: <l1> with stab index 1
    if EAQ and GL and R.auto = [ [ A[1][1], A[1][2] ], [ A[2][1], A[2][2] ] ]*z then 
        return [1];
    fi;

	# quotient elem. ab. dim 2 
	# orb reps: <l1> with stab index p-1
	#           <l2> with stab index 1
	if EAQ and GL and R.auto = [ [ A[1][1], A[1][2] ], [ 0, 1 ] ]*z then
		return [p];
	fi;

	# quotient has type C_{p^2} x C_p = <l1,l3> x <l2>
	# orb reps: <l1,l3> with stab index 1
	#           <l2,l3> with stab index 1
	#           <l2> with stab index 1
	if not EAQ and GL and ( R.auto = [ [ A[1][1], A[1][2], A[1][3] ], [ 0, A[2][2], A[2][3] ] ]*z or
							R.auto = [ [ A[1][1], A[1][2], A[1][4] ], [ 0, A[2][2], A[2][4] ] ]*z ) then
		return [2,1];
	fi;

	# quotient has type C_{p^2} x C_{p^2} = <l1,l3> x <l2,l4>
	# orb reps: <l2,l3,l4> with stab index 1
	#           <l2,l4> with stab index 1
	if not EAQ and GL and ( R.auto =  [ [ A[1][1], A[1][2], A[1][3], A[1][4] ], [ A[2][1], A[2][2], A[2][3], A[2][4] ] ]*z or 
                            R.auto =  [ [ A[1][1], A[1][2], A[1][4], A[1][5] ], [ A[2][1], A[2][2], A[2][4], A[2][5] ] ]*z ) then
		return [1,1];
	fi;

	# quotient has type C_{p^2} x C_p = <l1,l3> x <l2>
	# orb reps: <l1,l3> with stab index 1
	#           <l2,l3> with stab index p-1
	#           <l2>    with stab index p-1
	if not EAQ and GL and R.auto =  [ [ 1, A[1][2], A[1][4] ], [ 0, A[2][2], A[2][4] ] ]*z then
		return [p,p-1];
	fi;

	# quotient elem. ab. dim 2 
	# orb reps: <l1> with stab index 1
	#           <l2> with stab index 1
	if EAQ and GL and R.auto = [ [ A[1][1], A[1][2] ], [ 0, A[2][2] ] ]*z then
		return [2];
	fi;

	# quotient elem. ab. dim 2 
	# orb reps: <l1> with stab index 1
	#           <l2> with stab index 1
	if EAQ and SL and R.auto = [ [ A[1][1], A[1][2] ], [ 0, A[2][2] ] ]*z then
		return [2];
	fi;

	# quotient elem. ab. dim 2 
	# orb reps: <l1>    with stab index 1
	#           <l2>    with stab index (p-1)/2
	#           <l1+l2> with stab index (p-1)/2
	if EAQ and GL and R.auto = [ [ A[2][2]*D, 0 ], [ 0, A[2][2] ] ]*z then
		return [p];
	fi;

	# quotient has type C_{p^3} x C_p = <l1,l3,l4> x <l2>
	# orb reps: <l1,l3,l4> with stab index 1
	#           <l2,l3,l4> with stab index 1
    #           <l2,l4>    with stab index 1
    #           <l2+l3,l4> with stab index 1
    #           <l2>       with stab index 1
	if not EAQ and GL and ( R.auto =  [ [ A[1][1], A[1][2], A[1][3], A[1][4] ], [ 0, A[2][2], 0, A[2][4] ] ]*z or
                            R.auto =  [ [ A[1][1], A[1][2], A[1][4], A[1][5] ], [ 0, A[2][2], 0, A[2][5] ] ]*z) then
		return [2,2,1];
	fi;

	# quotient has type C_{p^2} x C_p = <l1,l3> x <l2>
	# orb reps: <l1,l3>    with stab index p-1
	#           <l2,l3>    with stab index 1
    #           <l2>       with stab index 1
    #           <l2+l3>    with stab index p-1
	if not EAQ and GL and R.auto =  [ [ A[1][1], A[1][2], A[1][3] ], [ 0, 1, 0 ] ]*z then
		return [p,p];
	fi;

	# quotient has type C_{p^2} x C_p x C_p = <l1,l4> x <l2> x <l3>
	# orb reps: <l1,l3,l4> with stab index 1
	#           <l2,l3,l4> with stab index 1
    #           <l2,l3>    with stab index 1
	if not EAQ and GL and R.auto =  [ [ A[1][1], A[1][2], A[1][3], A[1][4] ], [ 0, A[2][2], A[2][3], A[2][4] ], [ 0, A[3][2], A[3][3], A[3][4] ] ]*z then
		return [2,1];
	fi;

	# quotient elem. ab. dim 3 
	# orb reps: <l1,l2> with stab index 1
	#           <l2,l3> with stab index 1
	if EAQ and GL and R.auto = [ [ A[1][1], A[1][2], A[1][3] ], [ A[2][1], A[2][2], A[2][3] ], [ 0, 0, A[3][3] ] ]*z then
		return [2];
	fi;

	# quotient elem. ab. dim 3 
	# orb reps: <l1,l2> with stab index 1
	#           <l2,l3> with stab index 1
    #           <l1,l3> with stab index p-1
	if EAQ and GL and R.auto = [ [ A[1][1], A[1][2], A[1][3] ], [ 0, 1, A[2][3] ], [ 0, 0, A[3][3] ] ]*z then
		return [p+1];
	fi;

	# quotient elem. ab. dim 3 
	# orb reps: <l1,l2>    with stab index 1
	#           <l2,l3>    with stab index 1
    #           <l1,l2+l3> with stab index 1
	if EAQ and GL and R.auto = [ [ A[1][1], A[1][2], 0 ], [ A[2][1], A[2][2], 0 ], [ 0, 0, A[1][1]*A[2][2]-A[1][2]*A[2][1] ] ]*z then
		return [3];
	fi;

	# quotient elem. ab. dim 2
	# orb reps: <l1> with stab index (p-1)/2
	#           <l2> with stab index 1
	if EAQ and GL and R.auto = [ [ A[1][1], A[1][2] ], [ 0, A[1][1]*D ] ]*z then
		return [(p+1)/2];
	fi;

	# quotient elem. ab. dim 2
	# orb reps: <l1>                           with stab index 1
	#           <l2>                           with stab index 1
    #           <l1+xl2>, x not a square mod p with stab index (p-1)/2
    #           <l1+xl2>, x a square mod p     with stab index (p-1)/2
	if EAQ and SL and R.auto = [ [ A[1][1], 0 ], [ 0, A[2][2] ] ]*z then
		return [p+1];
	fi;

	# quotient elem. ab. dim 2
	# orb reps: <l1> with stab index (p-1)/2
	#           <l2> with stab index (p-1)/2
	if EAQ and SL and R.auto = [ [ A[1][1], A[1][2] ], [ 0, A[1][1] ] ]*z then
		return [p-1];
	fi;

	# RESTART
	# 5.12
	# quotient has type C_{p^3} x C_{p^2} = <l1,l3,l5> x <l2,l4>
	# orb reps: <l1,l3,l4,l5> with stab index 1
	#           <l2,l3,l4,l5> with stab index 1
    #           <l1,l3,l5>    with stab index 1
    #           <l2,l4,l5>    with stab index 1
    #           <l2,l4>       with stab index 1             
	if not EAQ and GL and R.auto =  [ [ A[1][1], A[1][2], A[1][3], A[1][4], A[1][5] ], [ 0, A[2][2], A[2][3], A[2][4], A[2][5] ] ]*z then
		return [2,2,1];
	fi;

	# 5.13
	# quotient has type C_{p^2} x C_{p^2} = <l1,l3> x <l2,l4>
	# orb reps: <l1,l3,l4,l5> with stab index p-1
	#           <l2,l3,l4,l5> with stab index 1
    #           <l1,l3,l5>    with stab index p-1
    #           <l2,l4,l5>    with stab index 1 
	if not EAQ and GL and R.auto =  [ [ A[1][1], A[1][2], A[1][3], A[1][4] ], [ 0, 1, A[2][3], A[2][4] ] ]*z then
		return [p,p];
	fi;

	# 5.16
	# quotient has type C_{p^2} x C_p = <l1,l3> x <l2>
	# orb reps: <l1,l3> with stab index 1
	#           <l2,l3> with stab index (p-1)/2
    #           <l2>    with stab index (p-1)/2
	if not EAQ and GL and R.auto =  [ [ A[2][2]*D, A[1][2], A[1][4] ], [ 0, A[2][2], A[2][4] ] ]*z then
		return [(p+1)/2,(p-1)/2];
	fi;

	# 5.19
	# quotient has type C_{p^2} x C_p = <l1,l3> x <l2>
	# orb reps: <l1,l3>    with stab index 1
	#           <l2,l3>    with stab index 1
	#           <l1+l2,l3> with stab index 1
    #           <l2>       with stab index 1
	if not EAQ and GL and R.auto =  [ [ A[1][1], 0, A[1][4] ], [ 0, A[2][2], A[2][4] ] ]*z then
		return [3,1];
	fi;

	# TODO: CHECK 4th ORB REP
	# 5.20
	# quotient has type C_{p^2} x C_p = <l1,l3> x <l2>
	# orb reps: <l1,l3>    with stab index 1
	#           <l2,l3>    with stab index 1
	#           <l1+l2,l3> with stab index (p-1)/2
	#           <l1+l2,l3> with stab index (p-1)/2
    #           <l2>       with stab index 1
	if not EAQ and SL and R.auto =  [ [ A[1][1], 0, A[1][4] ], [ 0, A[2][2], A[2][4] ] ]*z then
		return [p+1,1];
	fi;

	# 5.21
	# quotient has type C_{p^2} x C_p = <l1,l3> x <l2>
	# quotient has type C_{p^2} x C_p = <l1,l3> x <l2>
	# orb reps: <l1,l3>    with stab index (p-1)/2
	#           <l2,l3>    with stab index 1
    #           <l1+l2,l3> with stab index (p-1)/2
    #           <l2>       with stab index 1
    #           <l2+l3>    with stab index (p-1)/2
	if not EAQ and GL and R.auto =  [ [ A[1][1], 0, A[1][4] ], [ 0, A[1][1]*D, 0 ] ]*z then
		return [p,(p+1)/2];
	fi;

	# 5.23
	# quotient has type C_{p^3} x C_p = <l1,l3,l4> x <l2>
	# orb reps: <l1,l3,l4> with stab index 1
	#           <l2,l3,l4> with stab index p-1
    #           <l2,l4>    with stab index p-1
    #           <l2+l3,l4> with stab index p-1
    #           <l2>       with stab index p-1
	if not EAQ and GL and R.auto =  [ [ 1, A[1][2], A[1][4], A[1][5] ], [ 0, A[2][2], 0, A[2][5] ] ]*z then
		return [p,2*p-2,p-1];
	fi;

	# 5.24
	# quotient has type C_{p^2} x C_p = <l1,l3> x <l2>
	# orb reps: <l1,l3>    with stab index 1
	#           <l2,l3>    with stab index p-1
	#           <l1+l2,l3> with stab index p-1
    #           <l2>       with stab index p^2-p
	if not EAQ and GL and R.auto =  [ [ 1, 0, 0 ], [ 0, A[2][2], A[2][4] ] ]*z then
		return [2*p-1,p^2-p];
	fi;

	# 5.26
	# quotient elem. ab. dim 2
	# orb reps: <l1> with stab index 2
    #           <l2> with stab index 1
	if EAQ and R.auto = [ [ A[1][1], A[1][2] ], [0, A[2][2] ] ]*z and
			   R.eqns = [ A[1][1]^2*A[2][2] - 1 ] then
		return [3];
	fi;

	# 5.27
	# quotient elem. ab. dim 2
	# orb reps: <l1,l3>    with stab index 1
	#           <l2,l3>    with stab index (p-1)/Gcd(p-1,3)
	#           <l1+l2,l3> with stab index (p-1)/Gcd(p-1,3)
	if EAQ and GL and R.auto = [ [ A[2][2]^2*D^2, 0 ], [0, A[2][2] ] ]*z and LibraryConditions(L) = ["", ""] then
		return [1+2*(p-1)/g3];
	fi;

	# 5.28 / 5.29, 5.27 + p=1 mod 3
	# quotient elem. ab. dim 2
	if EAQ and GL and R.auto = [ [ A[2][2]^2*D^2, 0 ], [0, A[2][2] ] ]*z then
		return [((g3-1)/2)*(1+2*(p-1)/g3)];
	fi;

	# 5.30
	# quotient elem. ab. dim 2
	# orb reps: <l1> with stab index 2
    #           <l2> with stab index 1
	if EAQ and GL and R.auto = [ [ A[2][2]^2*D, A[1][2] ], [0, A[2][2] ] ]*z then
		return [3];
	fi;

	# 5.35
	# quotient elem. ab. dim 2
	if EAQ and SL and R.auto = [ [ A[2][2]^2, 0 ], [0, A[2][2] ] ]*z and LibraryConditions(L) = ["",""] then
		return [(p^2-1)/g3];
	fi;

	# 5.36 / 5.37, 5.35 + p=1 mod 3
	# quotient elem. ab. dim 2
	if EAQ and SL and R.auto = [ [ A[2][2]^2, 0 ], [0, A[2][2] ] ]*z then
		return [((g3-1)/2)*((p^2-1)/g3)];
	fi;

	# 5.38
	# quotient has type C_{p^4} x C_p = <l1,l3,l4,l5> x <l2>
	# orb reps: <l1,l3,l4,l5> with stab index 1
    #           <l2,l3,l4,l5> with stab index 1
	#           <l2,l4,l5>    with stab index 1
    #           <l2+l3,l4,l5> with stab index 1
    #           <l2,l5>       with stab index 1
    #           <l2+l4,l5>    with stab index 1
    #           <l2>          with stab index 1
	if not EAQ and GL and R.auto =  [ [ A[1][1], A[1][2], A[1][3], A[1][4], A[1][5] ], [ 0, A[2][2], 0, 0, A[2][5] ] ]*z then
		return [2,2,2,1];
	fi;

	# 5.39
	# quotient has type C_{p^3} x C_p = <l1,l3,l4> x <l2>
	# orb reps: <l1,l3,l4> with stab index p-1
    #           <l2,l3,l4> with stab index 1
	#           <l2,l4>    with stab index 1
    #           <l2+l3,l4> with stab index p-1
    #           <l2+l4>    with stab index p-1
    #           <l2>       with stab index 1
	if not EAQ and GL and R.auto =  [ [ A[1][1], A[1][2], A[1][3], A[1][4] ], [ 0, 1, 0, 0 ] ]*z then
		return [p,p,p];
	fi;

	# 5.40
	# quotient has type C_{p^2} x C_{p^2} x C_p = <l1,l4> x <l2,l5> x <l3>
	# orb reps: <l1,l3,l4,l5> with stab index 1
    #           <l1,l2,l4,l5> with stab index 1
    #           <l1,l3,l4>    with stab index 1
	if not EAQ and GL and R.auto =  [ [ A[1][1], A[1][2], A[1][3], A[1][4], A[1][5] ], 
                                      [ A[2][1], A[2][2], A[2][3], A[2][4], A[2][5] ], 
                                      [ 0, 0, A[3][3], A[3][4], A[3][5] ] ]*z then
		return [2,1];
	fi;

	# 5.41
	# quotient has type C_{p^2} x C_p x C_p = <l1,l4> x <l2> x <l3>
	# orb reps: <l1,l2,l4> with stab index 1
    #           <l1,l3,l4> with stab index 1
    #           <l2,l3,l4> with stab index 1
    #           <l2,l3>    with stab index 1
	if not EAQ and GL and R.auto =  [ [ A[1][1], A[1][2], A[1][3], A[1][5] ], 
                                      [ 0, A[2][2], A[2][3], A[2][5] ], 
                                      [ 0, 0, A[3][3], A[3][5] ] ]*z then
		return [3,1];
	fi;

	# 5.42
	# quotient has type C_{p^2} x C_p x C_p = <l1,l4> x <l2> x <l3>
	# orb reps: <l1,l3,l4> with stab index 1
    #           <l1,l2,l4> with stab index 1
    #           <l2,l3,l4> with stab index p-1
    #           <l2,l3>    with stab index p-1
	if not EAQ and GL and R.auto =  [ [ 1, A[1][2], A[1][3], A[1][5] ], 
                                      [ 0, A[2][2], A[2][3], A[2][5] ], 
                                      [ 0, 0, A[3][3], A[3][5] ] ]*z then
		return [p+1,p-1];
	fi;

	# 5.43
	# quotient has type C_{p^2} x C_p x C_p = <l1,l4> x <l2> x <l3>
	# orb reps: <l1,l3,l4>    with stab index 1
    #           <l1,l2,l4>    with stab index 1
    #           <l2,l3,l4>    with stab index 1
    #           <l1,l2+l3,l4> with stab index 1
    #           <l2,l3>       with stab index 1
	if not EAQ and GL and R.auto =  [ [ A[1][1], A[1][2], A[1][3], A[1][5] ], 
                                      [ 0, A[2][2], 0, A[2][5] ], 
                                      [ 0, 0, A[1][1]*A[2][2], A[3][5] ] ]*z then
		return [4,1];
	fi;

	# 5.44
	# quotient has type C_{p^2} x C_p x C_p = <l1,l2> x <l3> x <l4>
	# orb reps: <l1,l3,l4>    with stab index 1
    #           <l1,l2,l4>    with stab index 1
    #           <l1+l3,l2,l4> with stab index 1
    #           <l1,l2>       with stab index 1
	if not EAQ and GL and R.auto =  [ [ A[1][1], A[1][2], 0, A[1][5] ], 
                                      [ A[2][1], A[2][2], 0, A[2][5] ], 
                                      [ 0, 0, A[3][3], A[3][5] ] ]*z then
		return [3,1];
	fi;

	# 5.45
	# quotient has type C_{p^2} x C_p x C_p = <l1,l2> x <l3> x <l4>
	# orb reps: <l1,l3,l4>    with stab index p-1
    #           <l1,l2,l4>    with stab index 1
    #           <l1+l3,l2,l4> with stab index 1
    #           <l2,l3,l4>    with stab index 1
    #           <l1,l2+l3,l4> with stab index p-1
    #           <l1,l2>       with stab index 1
	if not EAQ and GL and R.auto =  [ [ A[1][1], A[1][2], 0, A[1][5] ], 
                                      [ 0, 1, 0, A[2][5] ], 
                                      [ 0, 0, A[3][3], A[3][5] ] ]*z then
		return [2*p+1,1];
	fi;

	# 5.46
	# quotient elem. ab. dim 3
	# orb reps: <l1,l2> with stab index 1
    #           <l2,l3> with stab index 1
	if EAQ and GL and R.auto = [ [ A[1][1], A[1][2], A[1][3] ], [0, A[2][2], A[2][3] ], [0, A[3][2], A[3][3] ] ]*z then
		return [2];
	fi;

	# 5.47
	# quotient elem. ab. dim 3
	# orb reps: <l1,l2> with stab index 1
    #           <l1,l3> with stab index p-1
    #           <l2,l3> with stab index 1
	if EAQ and GL and R.auto = [ [ A[1][1], A[1][2], A[1][3] ], [0, 1, 0 ], [0, A[3][2], A[3][3] ] ]*z then
		return [p+1];
	fi;

	# 5.48
	# quotient elem. ab. dim 3
	# orb reps: <l1,l2>    with stab index 1
    #           <l1,l3>    with stab index 1
    #           <l2,l3>    with stab index p-1
    #           <l1+l2,l3> with stab index p-1
    #           <l1,l2+l3> with stab index 1
	if EAQ and GL and R.auto = [ [ 1, 0, A[1][3] ], [0, A[2][2], 0 ], [0, 0, A[3][3] ] ]*z then
		return [2*p+1];
	fi;

	# 5.49
	# quotient elem. ab. dim 3
	# orb reps: <l1,l2>    with stab index p-1
    #           <l1,l3>    with stab index 1
    #           <l2,l3>    with stab index p-1
    #           <l1+l2,l3> with stab index p-1
    #           <l1,l2+l3> with stab index p-1
	if EAQ and GL and R.auto = [ [ 1, 0, A[1][3] ], [0, A[2][2], 0 ], [0, 0, 1 ] ]*z then
		return [4*p-3];
	fi;

	# 5.50
	# quotient elem. ab. dim 3
	# orb reps: <l1,l2>    with stab index 1
    #           <l1,l3>    with stab index 1
    #           <l2,l3>    with stab index 1
    #           <l1+l2,l3> with stab index 1
	if EAQ and GL and R.auto = [ [ A[1][1], 0, A[1][3] ], [0, A[1][1]*A[3][3], A[2][3] ], [0, 0, A[3][3] ] ]*z then
		return [4];
	fi;

	# 5.51
	# quotient elem. ab. dim 3
	# orb reps: <l1,l2>    with stab index 1
    #           <l1,l3>    with stab index p-1
    #           <l2,l3>    with stab index 1
	if EAQ and SL and R.auto = [ [ A[1][1], A[1][2], A[1][3] ], [0, 1, A[1][2]*A[3][3] ], [0, 0, A[3][3] ] ]*z then
		return [p+1];
	fi;

	# 5.53
	# quotient elem. ab. dim 3
	# orb reps: <l1,l2>    with stab index 1
    #           <l1,l3>    with stab index 1
    #           <l2,l3>    with stab index p-1
    #           <l1+l2,l3> with stab index p-1
    #           <l1+l3,l2> with stab index p-1
	if EAQ and GL and R.auto = [ [ 1, 0, 0 ], [0, A[2][2], A[2][3] ], [0, 0, A[2][2] ] ]*z then
		return [3*p-1];
	fi;

	# 5.55
	# quotient elem. ab. dim 3, (p-1)/2 cases
	# orb reps: <l1,l3>    with stab index 1
    #           <l1+l3,l2> with stab index p-1
    #           <l2,l3>    with stab index p-1
	if EAQ and GL and R.auto = [ [ 1, 0, 0 ], [0, A[2][2], A[3][2]*x ], [0, A[3][2], A[2][2]+A[3][2] ] ]*z then
		return [((p-1)/2)*(2*p-1)];
	fi;

	# 5.56
	# quotient has type C_{p^3} x C_p x C_p = <l1,l4,l5> x <l2> x <l3>
	# orb reps: <l1,l3,l4,l5> with stab index 1
    #           <l2,l3,l4,l5> with stab index 1
    #           <l2+l4,l3,l5> with stab index 1
    #           <l2,l3,l5>    with stab index 1
    #           <l2,l3>       with stab index 1
	if not EAQ and GL and R.auto =  [ [ A[1][1], A[1][2], A[1][3], A[1][4], A[1][5] ], 
                                      [ 0, A[2][2], A[2][3], 0, A[2][5] ], 
                                      [ 0, A[3][2], A[3][3], 0, A[3][5] ] ]*z then
		return [2,2,1];
	fi;

	# 5.57
	# quotient has type C_{p^2} x C_p x C_p = <l1,l4> x <l2> x <l3>
	# orb reps: <l1,l3,l4>    with stab index 1
    #           <l1+l3,l2,l4> with stab index 1
    #           <l2,l3,l4>    with stab index 1
    #           <l2+l4,l3>    with stab index 1
    #           <l2,l3>       with stab index 1
	if not EAQ and GL and R.auto =  [ [ A[2][2]*A[3][3]-A[2][3]*A[3][2], 0, 0, A[1][4] ], 
                                      [ 0, A[2][2], A[2][3], 0 ], 
                                      [ 0, A[3][2], A[3][3], 0 ] ]*z then
		return [3,2];
	fi;

	# 5.58
	# quotient has type C_{p^2} x C_p x C_p = <l1,l4> x <l2> x <l3>
	# orb reps: <l1,l2,l4> with stab index 1
    #           <l1,l3,l4> with stab index p-1
    #           <l2,l3,l4> with stab index 1
    #           <l2,l3+l4> with stab index 1
    #           <l2+l4,l3> with stab index p-1
    #           <l2,l3>    with stab index 1
	if not EAQ and GL and R.auto =  [ [ A[1][1], A[1][2], A[1][3], A[1][4] ], 
                                      [ 0, 1, A[2][3], 0 ], 
                                      [ 0, 0, A[3][3], 0 ] ]*z then
		return [p+1,p+1];
	fi;

	# 5.59
	# quotient elem. ab. dim 3
	# orb reps: <l1,l2> with stab index 1
    #           <l1,l3> with stab index 1
    #           <l2,l3> with stab index 1
	if EAQ and GL and R.auto = [ [ A[1][1], A[1][2], A[1][3] ], [0, A[2][2], A[2][3] ], [0, 0, A[3][3] ] ]*z then
		return [3];
	fi;

	# 5.60
	# quotient elem. ab. dim 3
	# orb reps: <l1,l2> with stab index 1
    #           <l1,l3> with stab index 1
    #           <l2,l3> with stab index 1
	if EAQ and R.auto = [ [ A[1][1], A[1][2], A[1][3] ], [0, A[2][2], A[2][3] ], [0, 0, A[3][3] ] ]*z 
           and R.eqns = [A[3][3]*D-1, A[1][1]*A[2][2]-1] then
		return [3];
	fi;

	# 5.61
	# quotient elem. ab. dim 3
	# orb reps: <l1,l2>    with stab index 1
    #           <l1,l3>    with stab index 1
    #           <l2,l3>    with stab index (p-1)/2
    #           <l1+l2,l3> with stab index (p-1)/2
	if EAQ and GL and R.auto = [ [ A[2][2]*A[3][3]*D, 0, A[1][3] ], [0, A[2][2], A[2][3] ], [0, 0, A[3][3] ] ]*z then
		return [p+1];
	fi;

	# TODO: Check last orbit rep? square/nonsquare?!
	# 5.63
	# quotient elem. ab. dim 3
	# orb reps: <l1,l2>    with stab index 1
    #           <l1,l3>    with stab index 1
    #           <l2,l3>    with stab index 1
    #           <l1+l3,l2> with stab index 1
    #           <l1,l2+l3> with stab index 1
    #           <l1,yl2+l3> with stab index 1
	if EAQ and GL and R.auto = [ [ A[3][3]^2*D, A[1][2], 0 ], [0, A[2][2], 0 ], [0, 0, A[3][3] ] ]*z then
		return [6];
	fi;

	# 5.64
	# quotient elem. ab. dim 3
	# orb reps: <l1,l2> with stab index 2
    #           <l1,l3> with stab index 1
    #           <l2,l3> with stab index 1
	if EAQ and GL and R.auto = [ [ A[2][2]*A[3][3]^2*D, -A[2][2]*A[3][3]*A[3][4]*D, A[1][3] ], [0, A[2][2], A[2][3] ], [0, 0, A[3][3] ] ]*z then
		return [4];
	fi;

	# 5.65
	# quotient elem. ab. dim 3
	# orb reps: <l1,l2> with stab index 2
    #           <l1,l3> with stab index 1
    #           <l2,l3> with stab index 1
	if EAQ and R.auto = [ [ A[2][2]*A[3][3], -A[2][2]*A[3][4], A[1][3] ], [0, A[2][2], A[2][3] ], [0, 0, A[3][3] ] ]*z and
               R.eqns = [ A[2][2]^2*A[3][3]-1 ] then
		return [4];
	fi;

	# 5.66
	# quotient elem. ab. dim 3
	# orb reps: <l1,l2>    with stab index p-1
    #           <l1,l3>    with stab index 1
    #           <l2,l3>    with stab index (p-1)/2
    #           <l1+l2,l3> with stab index (p-1)/2
	if EAQ and GL and R.auto = [ [ A[2][2]*D, 0, A[1][3] ], [0, A[2][2], A[2][3] ], [0, 0, 1 ] ]*z then
		return [2*p-1];
	fi;

	# TODO: Check last orbit rep.
	# 5.68
	# quotient elem. ab. dim 3
	# orb reps: <l1,l2>    with stab index 2
    #           <l1,l3>    with stab index p-1
    #           <l2,l3>    with stab index 1
    #           <l1+l3,l2> with stab index p-1
    #           <l1,l2+l3> with stab index p-1
    #           <l1,yl2+l3> with stab index p-1
	if EAQ and GL and R.auto = [ [ A[3][3]^2*D, -A[3][3]*A[3][4]*D, 0 ], [0, 1, 0 ], [0, 0, A[3][3] ] ]*z then
		return [4*p-1];
	fi;

	# 5.69
	# quotient has type C_{p^2} x C_p x C_p x C_p = <l1,l5> x <l2> x <l3> x <l4>
	# orb reps: <l1,l2,l4,l5> with stab index 1
    #           <l2,l3,l4,l5> with stab index 1
    #           <l2,l3,l4>    with stab index 1
	if not EAQ and GL and R.auto =  [ [ A[1][1], A[1][2], A[1][3], A[1][4], A[1][5] ], 
                                      [ 0, A[2][2], A[2][3], A[2][4], A[2][5] ], 
                                      [ 0, A[3][2], A[3][3], A[3][4], A[3][5] ],
                                      [ 0, A[4][2], A[4][3], A[4][4], A[4][5] ] ]*z then
		return [2,1];
	fi;

	# 5.70
	# quotient elem. ab. dim 4
	# orb reps: <l1,l2,l4> with stab index 1
    #           <l1,l3,l4> with stab index 1
	if EAQ and GL and R.auto =  [ [ A[1][1], A[1][2], A[1][3], A[1][4] ], 
                                  [ A[2][1], A[2][2], A[2][3], A[2][4] ], 
                                  [ 0, 0, A[3][3], A[3][4] ],
                                  [ 0, 0, A[4][3], A[4][4] ] ]*z then
		return [2];
	fi;

	# 5.71
	# quotient elem. ab. dim 4
	# orb reps: <l1,l2,l4> with stab index 1
    #           <l1,l3,l4> with stab index p-1
    #           <l2,l3,l4> with stab index 1
	if EAQ and GL and R.auto =  [ [ A[1][1], A[1][2], A[1][3], A[1][4] ], 
                                  [ 0, 1, A[2][3], A[2][4] ], 
                                  [ 0, 0, A[3][3], A[3][4] ],
                                  [ 0, 0, A[4][3], A[4][4] ] ]*z then
		return [p+1];
	fi;

	# 5.72
	# quotient elem. ab. dim 4
	# orb reps: <l1,l2,l3>    with stab index 1
    #           <l1,l2,l4>    with stab index 1
    #           <l1,l3,l4>    with stab index 1
    #           <l1+l3,l2,l4> with stab index 1
	if EAQ and GL and R.auto =  [ [ A[1][1], A[1][2], 0, A[1][4] ], 
                                  [ A[2][1], A[2][2], 0, A[2][4] ], 
                                  [ 0, 0, A[1][1]*A[2][2]-A[1][2]*A[2][1], A[3][4] ],
                                  [ 0, 0, 0, A[4][4] ] ]*z then
		return [4];
	fi;

	# 5.74
	# quotient elem. ab. dim 4
	# orb reps: <l1,l2,l4>    with stab index 1
    #           <l1,l2+l4,l3> with stab index p-1
    #           <l2,l3,l4>    with stab index 1
	if EAQ and GL and R.auto =  [ [ A[3][3]*A[4][4]-A[3][4]*A[4][3], A[1][2], A[3][2]*A[4][3]-A[3][3]*A[4][2], A[3][2]*A[4][4]-A[3][4]*A[4][2] ], 
                                  [ 0, 1, 0, 0 ], 
                                  [ 0, A[3][2], A[3][3], A[3][4] ],
                                  [ 0, A[4][2], A[4][3], A[4][4] ] ]*z then
		return [p+1];
	fi;

	
	### SPECIAL CASES
	# 5.10
    # 2 orb reps with stab indices (p-1)/2, (p-1)/2
    # groups are dihedral groups of order 2p+2 and stabilizers all have size 4
    # similar to paper ex. 6
	if LibraryName(L) = "5.45" then
		return [p-1];
	fi;

	# 5.54
    # 3 orb reps with stab indices 1, (p-1)/2, (p-1)/2
	if LibraryName(L) = "5.22" then
		return [p];
	fi;

	# 5.73 
    # quotient elem. ab. dim 4, equations ugly (!!!)
	# interestingly automorphism group seems to be GL(4,q) 
	if LibraryName(L) = "5.6" then
		return [1];
	fi;

	# 5.11
	# see paper ex. 6
	if LibraryName(L) = "5.46" then
		return [((p-1)/2)*(p-1)];
	fi;

	# 5.18
	# see paper ex. 7
	if LibraryName(L) = "5.53" then
		return [2,p];
	fi;

	# 5.31
	# see paper ex. 8
	if LibraryName(L) in ["5.66", "5.67"] then
		return [3*(p-1)/g4];
	fi;

	# see paper ex. 8 + p=1 mod 4
	if LibraryName(L) in ["5.68", "5.69"] then
		return [((g4/2)-1)*(3*(p-1)/g4)];
	fi;

	### SPECIAL CASES THAT SPLIT INTO MULTIPLE CASES
	if LibraryName(L) = "5.20" then
		return SumOfIndices520();
	fi;

	# 5.7
	# see paper ex. 5
	if LibraryName(L) = "5.42" then
		return SumOfIndices542();
	fi;

    return "case not found";
end;


#############################################################################
##
## NumberPNQNormalQSylow(n) determines Q(p^nq) q <> p and n <= 5.
##
InstallMethod( NumberPNQNormalQSylow, [IsPosInt], function( n )
    local OLDORDER, L, l, a, i, b, c;

    OLDORDER := ORDER;
    ORDER := MonomialLexOrdering();

	if not n in [1..5] then
		Error("n must be an integer between 1 and 5!");
	fi;

    L := LiePRingsByLibrary(n);
    a := 0;
    for i in [1..Length(L)] do
        l := LibraryName(L[i]);
        if l = "5.20" then 
            a := a + SumOfIndices520();
        elif l = "5.42" then 
            a := a + SumOfIndices542();
        else
            #b := NumberOfLiePRingsInFamily(L[i]);
            c := SumOfIndicesOfLiePRingNC(L[i]);
            a := a + c;
        fi;
    od;

	a := [DP, DP2, DP3, DP4, DP5]*a;

	# Replace gcds	
	a := Value(a, [g3], [2*D9+1]);
	a := Value(a, [g4], [P2+2*D10+2*D11]);

    # correction term for q = 2 and q = 3 
    if n = 4 then 
        a := a - P2 * DP;
    fi;

	if n = 5 then
		a := a - (36*P2 + 5*P3)*DP + P2*DP2;
	fi;

    ORDER := OLDORDER;

    return a;
end );

