# complete.gi
#############################################################################
##
#O  NumberPNQ( <n> ) . . . . . . . . . . . . . . . . . . . . .
##
##  <#GAPDoc Label="NumberPNQ">
##  <ManSection>
##  <Oper Name="NumberPNQ" Arg="n"/>
##  
##  <Description>
##  Given an integer <M>n \leq 5</M>, this returns a generalized PORC
##  polynomial in <M>p</M> and <M>q</M> enumerating the number
##  of isomorphism types of groups of order <M>p^nq</M>.
##  </Description>
##  </ManSection>
##  <#/GAPDoc>
DeclareOperation( "NumberPNQ", [IsPosInt] );


# exceptional.gi
#############################################################################
##
#O  NumberPNQExceptional( <n> ) . . . . . . . . . . . . . . . . . . . . .
##
##  <#GAPDoc Label="NumberPNQExceptional">
##  <ManSection>
##  <Oper Name="NumberPNQExceptional" Arg="n"/>
##  
##  <Description>
##  Given an integer <M>n \leq 5</M>, this returns a generalized PORC
##  polynomial in <M>p</M> and <M>q</M> enumerating the number
##  of isomorphism types of non-nilpotent groups of order <M>p^nq</M>, 
##  which do not contain a normal Sylow subgroup.
##  </Description>
##  </ManSection>
##  <#/GAPDoc>
DeclareOperation( "NumberPNQExceptional", [IsPosInt] );


# nilpotent.gi
#############################################################################
##
#O  NumberPNQNilpotent( <n> ) . . . . . . . . . . . . . . . . . . . . .
##
##  <#GAPDoc Label="NumberPNQNilpotent">
##  <ManSection>
##  <Oper Name="NumberPNQNilpotent" Arg="n"/>
##  
##  <Description>
##  Given an integer <M>n \leq 5</M>, this returns a generalized PORC
##  polynomial in <M>p</M> and <M>q</M> enumerating the number
##  of isomorphism types of nilpotent groups of order <M>p^nq</M>.
##  This coincides with the number of isomorphism types of groups of
##  order <M>p^n</M>.
##  </Description>
##  </ManSection>
##  <#/GAPDoc>
DeclareOperation( "NumberPNQNilpotent", [IsPosInt] );


# normalp.gi
#############################################################################
##
#O  NumberPNQNormalPSylow( <n> ) . . . . . . . . . . . . . . . . . . . . .
##
##  <#GAPDoc Label="NumberPNQNormalPSylow">
##  <ManSection>
##  <Oper Name="NumberPNQNormalPSylow" Arg="n"/>
##  
##  <Description>
##  Given an integer <M>n \leq 5</M>, this returns a generalized PORC
##  polynomial in <M>p</M> and <M>q</M> enumerating the number
##  of isomorphism types of non-nilpotent groups of order <M>p^nq</M>
##  with normal Sylow <M>p</M>-subgroup.
##  </Description>
##  </ManSection>
##  <#/GAPDoc>
DeclareOperation( "NumberPNQNormalPSylow", [IsPosInt] );


# normalq.gi
#############################################################################
##
#O  NumberPNQNormalQSylow( <n> ) . . . . . . . . . . . . . . . . . . . . .
##
##  <#GAPDoc Label="NumberPNQNormalQSylow">
##  <ManSection>
##  <Oper Name="NumberPNQNormalQSylow" Arg="n"/>
##  
##  <Description>
##  Given an integer <M>n \leq 5</M>, this returns a generalized PORC
##  polynomial in <M>p</M> and <M>q</M> enumerating the number
##  of isomorphism types of non-nilpotent groups of order <M>p^nq</M>
##  with normal Sylow <M>q</M>-subgroup.
##  </Description>
##  </ManSection>
##  <#/GAPDoc>
DeclareOperation( "NumberPNQNormalQSylow", [IsPosInt] );


# porc.gi
#############################################################################
##
#O  EvalPNQPorcPoly( <poly>, <p>, <q> ) . . . . . . . . . . . . . . . . . . . . .
##
##  <#GAPDoc Label="EvalPNQPorcPoly">
##  <ManSection>
##  <Oper Name="EvalPNQPorcPoly" Arg="poly, p, q"/>
##  
##  <Description>
##  This evaluates a generalized PORC polynomial returned by one of the other
##  functions of this package at the given primes <M>p</M> and <M>q</M>.
##  </Description>
##  </ManSection>
##  <#/GAPDoc>
DeclareOperation( "EvalPNQPorcPoly", [IsPolynomial, IsPosInt, IsPosInt] );
